import { BigNumber } from "ethers";
import { ThresholdBoundType, FlowTupleParameters } from "./UBAFeeTypes";
type ChainId = number;
type RouteCombination = string;
type ChainTokenCombination = string;
type DefaultOverrideStructure<PrimaryValue, OverrideKeyType extends string | number | symbol> = {
    default: PrimaryValue;
    override?: Record<OverrideKeyType, PrimaryValue>;
};
declare class UBAConfig {
    protected baselineFee: DefaultOverrideStructure<BigNumber, RouteCombination>;
    protected balancingFee: DefaultOverrideStructure<FlowTupleParameters, ChainId>;
    protected balanceTriggerThreshold: DefaultOverrideStructure<ThresholdBoundType, ChainTokenCombination>;
    protected lpGammaFunction: DefaultOverrideStructure<FlowTupleParameters, ChainId>;
    protected incentivePoolAdjustment: Record<string, BigNumber>;
    protected ubaRewardMultiplier: Record<string, BigNumber>;
    constructor(baselineFee: DefaultOverrideStructure<BigNumber, RouteCombination>, balancingFee: DefaultOverrideStructure<FlowTupleParameters, ChainId>, balanceTriggerThreshold: DefaultOverrideStructure<ThresholdBoundType, ChainTokenCombination>, lpGammaFunction: DefaultOverrideStructure<FlowTupleParameters, ChainId>, incentivePoolAdjustment?: Record<string, BigNumber>, ubaRewardMultiplier?: Record<string, BigNumber>);
    private assertValidityOfAllFeeCurves;
    getBaselineFee(destinationChainId: number, originChainId: number): BigNumber;
    getBalancingFeeTuples(chainId: number): FlowTupleParameters;
    getZeroFeePointOnBalancingFeeCurve(chainId: number): BigNumber;
    isBalancingFeeCurveFlatAtZero(chainId: number): boolean;
    getLpGammaFunctionTuples(chainId: number): FlowTupleParameters;
    getBalanceTriggerThreshold(chainId: number, tokenSymbol: string): ThresholdBoundType;
    getTargetBalance(chainId: number, tokenSymbol: string): BigNumber;
    getIncentivePoolAdjustment(chainId: string): BigNumber;
    getUbaRewardMultiplier(chainId: string): BigNumber;
    toJSON(): {
        baselineFee: any;
        balancingFee: any;
        balanceTriggerThreshold: any;
        lpGammaFunction: any;
        incentivePoolAdjustment: any;
        ubaRewardMultiplier: any;
    };
}
export default UBAConfig;
