"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseUBAConfigFromOnChain = exports.isUBAOnChainConfig = void 0;
var ethers_1 = require("ethers");
var superstructs_1 = require("./superstructs");
function isUBAOnChainConfig(input) {
    return superstructs_1.UBA_CONFIG_ONCHAIN_SCHEMA.is(input);
}
exports.isUBAOnChainConfig = isUBAOnChainConfig;
function parseUBAConfigFromOnChain(config) {
    var _a, _b;
    if (!isUBAOnChainConfig(config)) {
        throw new Error("No config provided");
    }
    return {
        incentivePoolAdjustment: Object.fromEntries(Object.entries((_a = config.incentivePoolAdjustment) !== null && _a !== void 0 ? _a : {}).map(function (_a) {
            var key = _a[0], value = _a[1];
            return [key, ethers_1.BigNumber.from(value)];
        })),
        ubaRewardMultiplier: Object.fromEntries(Object.entries((_b = config.ubaRewardMultiplier) !== null && _b !== void 0 ? _b : {}).map(function (_a) {
            var key = _a[0], value = _a[1];
            return [key, ethers_1.BigNumber.from(value)];
        })),
        alpha: Object.fromEntries(Object.entries(config.alpha).map(function (_a) {
            var key = _a[0], value = _a[1];
            return [key, ethers_1.BigNumber.from(value)];
        })),
        gamma: Object.fromEntries(Object.entries(config.gamma).map(function (_a) {
            var key = _a[0], value = _a[1];
            return [
                key,
                value.map(function (tuple) { return tuple.map(function (value) { return ethers_1.BigNumber.from(value); }); }),
            ];
        })),
        omega: Object.fromEntries(Object.entries(config.omega).map(function (_a) {
            var key = _a[0], value = _a[1];
            return [
                key,
                value.map(function (tuple) { return tuple.map(function (value) { return ethers_1.BigNumber.from(value); }); }),
            ];
        })),
        rebalance: Object.fromEntries(Object.entries(config.rebalance).map(function (_a) {
            var key = _a[0], _b = _a[1], threshold_lower = _b.threshold_lower, threshold_upper = _b.threshold_upper, target_lower = _b.target_lower, target_upper = _b.target_upper;
            var upperExists = threshold_upper !== undefined && target_upper !== undefined;
            var lowerExists = threshold_lower !== undefined && target_lower !== undefined;
            return [
                key,
                {
                    threshold_lower: lowerExists ? ethers_1.BigNumber.from(threshold_lower) : undefined,
                    target_lower: lowerExists ? ethers_1.BigNumber.from(target_lower) : undefined,
                    threshold_upper: upperExists ? ethers_1.BigNumber.from(threshold_upper) : undefined,
                    target_upper: upperExists ? ethers_1.BigNumber.from(target_upper) : undefined,
                },
            ];
        })),
    };
}
exports.parseUBAConfigFromOnChain = parseUBAConfigFromOnChain;
//# sourceMappingURL=ConfigStoreParsingUtilities.js.map