import { HubPoolClient } from "../HubPoolClient";
import UBAFeeConfig from "../../UBAFeeCalculator/UBAFeeConfig";
import { SpokePoolClients } from "../../utils/TypeUtils";
import { ModifiedUBAFlow, RequestValidReturnType, SpokePoolFillFilter, UBAClientState } from "./UBAClientTypes";
import { DepositWithBlock, Fill, FillWithBlock, RefundRequestWithBlock, TokenRunningBalance, UBAParsedConfigType, UbaFlow } from "../../interfaces";
export declare function parseUBAFeeConfig(chainId: number, tokenSymbol: string, ubaConfig?: UBAParsedConfigType): UBAFeeConfig | undefined;
export declare function getUBAFeeConfig(hubPoolClient: HubPoolClient, chainId: number, tokenSymbol: string, blockNumber?: number): UBAFeeConfig;
export declare function getMostRecentBundleBlockRanges(chainId: number, maxBundleStates: number, hubPoolClient: HubPoolClient, spokePoolClients: SpokePoolClients): {
    start: number;
    end: number;
}[];
export declare function getOpeningRunningBalanceForEvent(hubPoolClient: HubPoolClient, eventBlock: number, eventChain: number, l1Token: string, hubPoolLatestBlock: number): TokenRunningBalance;
export declare function getMatchedDeposit(spokePoolClients: SpokePoolClients, fill: Fill, fillFieldsToIgnore?: string[]): Promise<DepositWithBlock | undefined>;
export declare function isUBAActivatedAtBlock(hubPoolClient: HubPoolClient, block: number, chain: number): boolean;
export declare function getUbaActivationBundleStartBlocks(hubPoolClient: HubPoolClient): number[];
export declare function getFlowChain(flow: UbaFlow): number;
export declare function getUBAFlows(tokenSymbol: string, chainId: number, spokePoolClients: SpokePoolClients, hubPoolClient: HubPoolClient, fromBlock?: number, toBlock?: number): Promise<UbaFlow[]>;
export declare function getBundleKeyForBlockRanges(blockRanges: number[][]): string;
export declare function flowComparisonFunction(a: UbaFlow, b: UbaFlow): number;
export declare function sortFlowsAscendingInPlace(flows: UbaFlow[]): UbaFlow[];
export declare function sortFlowsAscending(flows: UbaFlow[]): UbaFlow[];
export declare function refundRequestIsValid(spokePoolClients: SpokePoolClients, hubPoolClient: HubPoolClient, refundRequest: RefundRequestWithBlock, ignoredDepositValidationParams?: string[]): Promise<RequestValidReturnType>;
export declare function getMatchingFlow(allValidatedFlows: ModifiedUBAFlow[], targetFlow: UbaFlow): ModifiedUBAFlow | undefined;
export declare function getValidFillCandidates(chainId: number, spokePoolClients: SpokePoolClients, filter?: SpokePoolFillFilter, ignoredDepositValidationParams?: string[]): Promise<(FillWithBlock & {
    matchedDeposit: DepositWithBlock;
})[]>;
export declare function getValidRefundCandidates(chainId: number, hubPoolClient: HubPoolClient, spokePoolClients: SpokePoolClients, filter?: Pick<SpokePoolFillFilter, "fromBlock" | "toBlock">, ignoredDepositValidationParams?: string[]): Promise<(RefundRequestWithBlock & {
    matchedDeposit: DepositWithBlock;
})[]>;
export declare function serializeUBAClientState(ubaClientState: UBAClientState): string;
export declare function deserializeUBAClientState(serializedUBAClientState: object): UBAClientState;
