"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deserializeUBAClientState = exports.serializeUBAClientState = exports.getValidRefundCandidates = exports.getValidFillCandidates = exports.getMatchingFlow = exports.refundRequestIsValid = exports.sortFlowsAscending = exports.sortFlowsAscendingInPlace = exports.flowComparisonFunction = exports.getBundleKeyForBlockRanges = exports.getUBAFlows = exports.getFlowChain = exports.getUbaActivationBundleStartBlocks = exports.isUBAActivatedAtBlock = exports.getMatchedDeposit = exports.getOpeningRunningBalanceForEvent = exports.getMostRecentBundleBlockRanges = exports.getUBAFeeConfig = exports.parseUBAFeeConfig = void 0;
var tslib_1 = require("tslib");
var assert_1 = tslib_1.__importDefault(require("assert"));
var ethers_1 = require("ethers");
var UBAFeeConfig_1 = tslib_1.__importDefault(require("../../UBAFeeCalculator/UBAFeeConfig"));
var ArrayUtils_1 = require("../../utils/ArrayUtils");
var TypeGuards_1 = require("../../utils/TypeGuards");
var FlowUtils_1 = require("../../utils/FlowUtils");
var interfaces_1 = require("../../interfaces");
var BundleUtils_1 = require("../../utils/BundleUtils");
var JSONUtils_1 = require("../../utils/JSONUtils");
var utils_1 = require("../../utils");
function omitDefaultKeys(obj) {
    return Object.keys(obj).reduce(function (acc, key) {
        var _a;
        if (key !== "default") {
            return tslib_1.__assign(tslib_1.__assign({}, acc), (_a = {}, _a[key] = obj[key], _a));
        }
        return acc;
    }, {});
}
function parseUBAFeeConfig(chainId, tokenSymbol, ubaConfig) {
    var _a;
    if (!ubaConfig) {
        return undefined;
    }
    var omegaDefault = ubaConfig.omega["default"];
    var omegaOverride = omitDefaultKeys(ubaConfig.omega);
    var gammaDefault = ubaConfig.gamma["default"];
    var gammaOverride = omitDefaultKeys(ubaConfig.gamma);
    var threshold = ubaConfig.rebalance[String(chainId)];
    var chainTokenCombination = "".concat(chainId, "-").concat(tokenSymbol);
    return new UBAFeeConfig_1.default({
        default: ubaConfig.alpha["default"],
        override: omitDefaultKeys(ubaConfig.alpha),
    }, {
        default: omegaDefault,
        override: omegaOverride,
    }, {
        default: {
            lowerBound: {
                target: (0, utils_1.toBN)(0),
                threshold: (0, utils_1.toBN)(0),
            },
            upperBound: {
                target: (0, utils_1.toBN)(0),
                threshold: (0, utils_1.toBN)(0),
            },
        },
        override: (_a = {},
            _a[chainTokenCombination] = {
                lowerBound: {
                    target: threshold === null || threshold === void 0 ? void 0 : threshold.target_lower,
                    threshold: threshold === null || threshold === void 0 ? void 0 : threshold.threshold_lower,
                },
                upperBound: {
                    target: threshold === null || threshold === void 0 ? void 0 : threshold.target_upper,
                    threshold: threshold === null || threshold === void 0 ? void 0 : threshold.threshold_upper,
                },
            },
            _a),
    }, {
        default: gammaDefault,
        override: gammaOverride,
    }, ubaConfig.incentivePoolAdjustment, ubaConfig.ubaRewardMultiplier);
}
exports.parseUBAFeeConfig = parseUBAFeeConfig;
function getUBAFeeConfig(hubPoolClient, chainId, tokenSymbol, blockNumber) {
    var configClient = hubPoolClient.configStoreClient;
    if (!configClient.isUpdated) {
        throw new Error("Config client not updated");
    }
    var l1TokenInfo = hubPoolClient.getL1Tokens().find(function (token) { return token.symbol === tokenSymbol; });
    if (!l1TokenInfo) {
        throw new Error("L1 token can't be found, have you updated hub pool client?");
    }
    var rawUbaConfig = configClient.getUBAConfig(l1TokenInfo === null || l1TokenInfo === void 0 ? void 0 : l1TokenInfo.address, blockNumber);
    var ubaConfig = parseUBAFeeConfig(chainId, tokenSymbol, rawUbaConfig);
    if (ubaConfig === undefined) {
        throw new Error("UBA config for blockTag ".concat(blockNumber, " not found"));
    }
    return ubaConfig;
}
exports.getUBAFeeConfig = getUBAFeeConfig;
function getMostRecentBundleBlockRanges(chainId, maxBundleStates, hubPoolClient, spokePoolClients) {
    var toBlock = hubPoolClient.latestBlockSearched;
    var ubaActivationStartBlocks = getUbaActivationBundleStartBlocks(hubPoolClient);
    var ubaActivationHubStartBlock = ubaActivationStartBlocks[0];
    var bundleData = [];
    for (var i = 0; i < maxBundleStates; i++) {
        var latestExecutedRootBundle = hubPoolClient.getNthFullyExecutedRootBundle(-1, toBlock);
        if (!latestExecutedRootBundle) {
            break;
        }
        var chainIndex = hubPoolClient.configStoreClient.getChainIdIndicesForBlock().indexOf(chainId);
        if (chainIndex >= latestExecutedRootBundle.bundleEvaluationBlockNumbers.length) {
            break;
        }
        var rootBundleBlockRanges = (0, BundleUtils_1.getImpliedBundleBlockRanges)(hubPoolClient, hubPoolClient.configStoreClient, latestExecutedRootBundle);
        var hubPoolStartBlock = (0, BundleUtils_1.getBlockRangeForChain)(rootBundleBlockRanges, hubPoolClient.chainId, hubPoolClient.configStoreClient.getChainIdIndicesForBlock())[0];
        if (hubPoolStartBlock < ubaActivationHubStartBlock) {
            break;
        }
        var blockRangeForChain = (0, BundleUtils_1.getBlockRangeForChain)(rootBundleBlockRanges, chainId, hubPoolClient.configStoreClient.getChainIdIndicesForBlock());
        bundleData.unshift({
            start: blockRangeForChain[0],
            end: blockRangeForChain[1],
        });
        toBlock = latestExecutedRootBundle.blockNumber;
    }
    if (bundleData.length === 0) {
        var ubaActivationBundleStartBlocks = getUbaActivationBundleStartBlocks(hubPoolClient);
        var ubaActivationBundleStartBlockForChain = (0, BundleUtils_1.getBlockForChain)(ubaActivationBundleStartBlocks, chainId, hubPoolClient.configStoreClient.getChainIdIndicesForBlock());
        bundleData.unshift({
            start: ubaActivationBundleStartBlockForChain,
            end: ubaActivationBundleStartBlockForChain,
        });
    }
    if ((0, TypeGuards_1.isDefined)(spokePoolClients[chainId])) {
        bundleData[bundleData.length - 1].end = spokePoolClients[chainId].latestBlockSearched;
    }
    return bundleData;
}
exports.getMostRecentBundleBlockRanges = getMostRecentBundleBlockRanges;
function getOpeningRunningBalanceForEvent(hubPoolClient, eventBlock, eventChain, l1Token, hubPoolLatestBlock) {
    var chainIdIndices = hubPoolClient.configStoreClient.getChainIdIndicesForBlock();
    var latestExecutedBundle = hubPoolClient.getNthFullyExecutedRootBundle(-1, hubPoolLatestBlock);
    if (!(0, TypeGuards_1.isDefined)(latestExecutedBundle)) {
        return {
            runningBalance: ethers_1.ethers.constants.Zero,
            incentiveBalance: ethers_1.ethers.constants.Zero,
        };
    }
    var chainIndex = chainIdIndices.indexOf(eventChain);
    if (chainIndex >= latestExecutedBundle.bundleEvaluationBlockNumbers.length) {
        return {
            runningBalance: ethers_1.ethers.constants.Zero,
            incentiveBalance: ethers_1.ethers.constants.Zero,
        };
    }
    var blockRanges = (0, BundleUtils_1.getImpliedBundleBlockRanges)(hubPoolClient, hubPoolClient.configStoreClient, latestExecutedBundle);
    var eventBlockRange = (0, BundleUtils_1.getBlockRangeForChain)(blockRanges, eventChain, chainIdIndices);
    if (eventBlock > eventBlockRange[1]) {
        var executedLeavesForBundle = hubPoolClient.getExecutedLeavesForRootBundle(latestExecutedBundle, hubPoolLatestBlock);
        if (executedLeavesForBundle.length === 0) {
            throw new Error("No executed leaves found for bundle");
        }
        var executedLeaf = executedLeavesForBundle.find(function (executedLeaf) { return executedLeaf.chainId === eventChain; });
        if (!executedLeaf) {
            return getOpeningRunningBalanceForEvent(hubPoolClient, eventBlock, eventChain, l1Token, latestExecutedBundle.blockNumber);
        }
        var l1TokenIndex = executedLeaf.l1Tokens.indexOf(l1Token);
        if (l1TokenIndex === -1) {
            return getOpeningRunningBalanceForEvent(hubPoolClient, eventBlock, eventChain, l1Token, latestExecutedBundle.blockNumber);
        }
        var runningBalance = executedLeaf.runningBalances[l1TokenIndex];
        var incentiveBalance = executedLeaf.incentiveBalances[l1TokenIndex];
        var ubaActivationStartBlocks = getUbaActivationBundleStartBlocks(hubPoolClient);
        var ubaActivationStartBlockForChain = (0, BundleUtils_1.getBlockForChain)(ubaActivationStartBlocks, eventChain, chainIdIndices);
        if (blockRanges[0][0] < ubaActivationStartBlockForChain) {
            return {
                runningBalance: runningBalance.mul(-1),
                incentiveBalance: ethers_1.ethers.constants.Zero,
            };
        }
        else {
            return {
                runningBalance: runningBalance,
                incentiveBalance: incentiveBalance,
            };
        }
    }
    return getOpeningRunningBalanceForEvent(hubPoolClient, eventBlock, eventChain, l1Token, latestExecutedBundle.blockNumber);
}
exports.getOpeningRunningBalanceForEvent = getOpeningRunningBalanceForEvent;
function getMatchedDeposit(spokePoolClients, fill, fillFieldsToIgnore) {
    if (fillFieldsToIgnore === void 0) { fillFieldsToIgnore = []; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var originSpokePoolClient, deposit;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    originSpokePoolClient = spokePoolClients[fill.originChainId];
                    if (!originSpokePoolClient.isUpdated) {
                        throw new Error("SpokePoolClient must be updated before querying historical deposits");
                    }
                    if (fill.depositId < originSpokePoolClient.firstDepositIdForSpokePool ||
                        fill.depositId > originSpokePoolClient.lastDepositIdForSpokePool) {
                        return [2, undefined];
                    }
                    if (fill.depositId >= originSpokePoolClient.earliestDepositIdQueried &&
                        fill.depositId <= originSpokePoolClient.latestDepositIdQueried) {
                        return [2, originSpokePoolClient.getDepositForFill(fill, fillFieldsToIgnore)];
                    }
                    return [4, originSpokePoolClient.findDeposit(fill.depositId, fill.destinationChainId, fill.depositor)];
                case 1:
                    deposit = _a.sent();
                    return [2, (0, FlowUtils_1.validateFillForDeposit)(fill, deposit, fillFieldsToIgnore) ? deposit : undefined];
            }
        });
    });
}
exports.getMatchedDeposit = getMatchedDeposit;
function isUBAActivatedAtBlock(hubPoolClient, block, chain) {
    var _a;
    try {
        var ubaActivationBlocks = getUbaActivationBundleStartBlocks(hubPoolClient);
        var chainIdIndices_1 = hubPoolClient.configStoreClient.getChainIdIndicesForBlock();
        var activationBlock = (_a = ubaActivationBlocks.find(function (_, idx) { return chainIdIndices_1[idx] === chain; })) !== null && _a !== void 0 ? _a : Number.MAX_SAFE_INTEGER;
        return block >= activationBlock;
    }
    catch (err) {
        return false;
    }
}
exports.isUBAActivatedAtBlock = isUBAActivatedAtBlock;
function getUbaActivationBundleStartBlocks(hubPoolClient) {
    var latestHubPoolBlock = hubPoolClient.latestBlockSearched;
    var chainIdIndices = hubPoolClient.configStoreClient.getChainIdIndicesForBlock();
    var ubaActivationBlock = hubPoolClient.configStoreClient.getUBAActivationBlock();
    if ((0, TypeGuards_1.isDefined)(ubaActivationBlock)) {
        var nextValidatedBundle_1 = hubPoolClient.getProposedRootBundles().find(function (bundle) {
            return bundle.blockNumber >= ubaActivationBlock
                ? hubPoolClient.isRootBundleValid(bundle, latestHubPoolBlock)
                : false;
        });
        if ((0, TypeGuards_1.isDefined)(nextValidatedBundle_1)) {
            var bundleBlockRanges_1 = (0, BundleUtils_1.getImpliedBundleBlockRanges)(hubPoolClient, hubPoolClient.configStoreClient, nextValidatedBundle_1);
            var bundleStartBlocks = chainIdIndices.map(function (chainId, i) {
                if (i >= nextValidatedBundle_1.bundleEvaluationBlockNumbers.length) {
                    return 0;
                }
                var startBlock = (0, BundleUtils_1.getBlockRangeForChain)(bundleBlockRanges_1, chainId, chainIdIndices)[0];
                return startBlock;
            });
            return bundleStartBlocks;
        }
        else {
            var nextBundleStartBlocks = chainIdIndices.map(function (chainId) {
                return hubPoolClient.getNextBundleStartBlockNumber(chainIdIndices, latestHubPoolBlock, chainId);
            });
            return nextBundleStartBlocks;
        }
    }
    else {
        throw new Error("UBA was not activated yet as of ".concat(latestHubPoolBlock));
    }
}
exports.getUbaActivationBundleStartBlocks = getUbaActivationBundleStartBlocks;
function getFlowChain(flow) {
    var flowChain;
    if ((0, interfaces_1.isUbaInflow)(flow)) {
        flowChain = flow.originChainId;
    }
    else {
        if ((0, interfaces_1.outflowIsFill)(flow)) {
            flowChain = flow.destinationChainId;
        }
        else {
            flowChain = flow.repaymentChainId;
        }
    }
    return flowChain;
}
exports.getFlowChain = getFlowChain;
function getUBAFlows(tokenSymbol, chainId, spokePoolClients, hubPoolClient, fromBlock, toBlock) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var spokePoolClient, deposits, preUBADeposit, fills, refundRequests, flows;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    spokePoolClient = spokePoolClients[chainId];
                    fromBlock = fromBlock !== null && fromBlock !== void 0 ? fromBlock : spokePoolClient.eventSearchConfig.fromBlock;
                    toBlock = toBlock !== null && toBlock !== void 0 ? toBlock : spokePoolClient.eventSearchConfig.toBlock;
                    deposits = spokePoolClient.getDeposits().filter(function (deposit) {
                        var _a;
                        if (deposit.blockNumber < fromBlock || deposit.blockNumber > toBlock) {
                            return false;
                        }
                        var _tokenSymbol = (_a = hubPoolClient.getL1TokenInfoForL2Token(deposit.originToken, deposit.originChainId)) === null || _a === void 0 ? void 0 : _a.symbol;
                        return _tokenSymbol === tokenSymbol;
                    });
                    preUBADeposit = deposits.find(function (deposit) { return (0, TypeGuards_1.isDefined)(deposit.realizedLpFeePct); });
                    if ((0, TypeGuards_1.isDefined)(preUBADeposit)) {
                        throw new Error("Found a UBA deposit with a defined realizedLpFeePct at block ".concat(preUBADeposit.blockNumber, " on chain ").concat(preUBADeposit.originChainId));
                    }
                    return [4, getValidFillCandidates(chainId, spokePoolClients, {
                            fromBlock: fromBlock,
                            toBlock: toBlock,
                            repaymentChainId: chainId,
                            isSlowRelay: false,
                            isCompleteFill: true,
                        }, ["realizedLpFeePct"])];
                case 1:
                    fills = (_a.sent()).filter(function (fill) {
                        var _a;
                        var _tokenSymbol = (_a = hubPoolClient.getL1TokenInfoForL2Token(fill.destinationToken, fill.destinationChainId)) === null || _a === void 0 ? void 0 : _a.symbol;
                        return fill.fillAmount.eq(fill.totalFilledAmount) && _tokenSymbol === tokenSymbol;
                    });
                    return [4, getValidRefundCandidates(chainId, hubPoolClient, spokePoolClients, {
                            fromBlock: fromBlock,
                            toBlock: toBlock,
                        }, ["realizedLpFeePct"])];
                case 2:
                    refundRequests = (_a.sent()).filter(function (refundRequest) {
                        var _a;
                        var _tokenSymbol = (_a = hubPoolClient.getL1TokenInfoForL2Token(refundRequest.refundToken, refundRequest.repaymentChainId)) === null || _a === void 0 ? void 0 : _a.symbol;
                        return _tokenSymbol === tokenSymbol;
                    });
                    flows = sortFlowsAscending(deposits.concat(fills).concat(refundRequests));
                    return [2, flows];
            }
        });
    });
}
exports.getUBAFlows = getUBAFlows;
function getBundleKeyForBlockRanges(blockRanges) {
    return JSON.stringify(blockRanges.map(function (blockRange) { return blockRange[0]; }));
}
exports.getBundleKeyForBlockRanges = getBundleKeyForBlockRanges;
function flowComparisonFunction(a, b) {
    if (a.blockTimestamp !== b.blockTimestamp) {
        return a.blockTimestamp - b.blockTimestamp;
    }
    if ((0, interfaces_1.isUbaInflow)(a) && (0, interfaces_1.isUbaOutflow)(b)) {
        return -1;
    }
    else if ((0, interfaces_1.isUbaInflow)(b) && (0, interfaces_1.isUbaOutflow)(a)) {
        return 1;
    }
    if (a.blockNumber !== b.blockNumber) {
        return a.blockNumber - b.blockNumber;
    }
    if (a.transactionIndex !== b.transactionIndex) {
        return a.transactionIndex - b.transactionIndex;
    }
    return a.logIndex - b.logIndex;
}
exports.flowComparisonFunction = flowComparisonFunction;
function sortFlowsAscendingInPlace(flows) {
    return flows.sort(function (fx, fy) { return flowComparisonFunction(fx, fy); });
}
exports.sortFlowsAscendingInPlace = sortFlowsAscendingInPlace;
function sortFlowsAscending(flows) {
    return sortFlowsAscendingInPlace(tslib_1.__spreadArray([], flows, true));
}
exports.sortFlowsAscending = sortFlowsAscending;
function refundRequestIsValid(spokePoolClients, hubPoolClient, refundRequest, ignoredDepositValidationParams) {
    if (ignoredDepositValidationParams === void 0) { ignoredDepositValidationParams = []; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var relayer, amount, refundToken, depositId, originChainId, destinationChainId, repaymentChainId, realizedLpFeePct, fillBlock, previousIdenticalRequests, destSpoke, deploymentBlock, latestBlockSearched, fill, deposit, expectedRefundToken;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    relayer = refundRequest.relayer, amount = refundRequest.amount, refundToken = refundRequest.refundToken, depositId = refundRequest.depositId, originChainId = refundRequest.originChainId, destinationChainId = refundRequest.destinationChainId, repaymentChainId = refundRequest.repaymentChainId, realizedLpFeePct = refundRequest.realizedLpFeePct, fillBlock = refundRequest.fillBlock, previousIdenticalRequests = refundRequest.previousIdenticalRequests;
                    if (destinationChainId === repaymentChainId) {
                        return [2, { valid: false, reason: "Invalid destinationChainId" }];
                    }
                    destSpoke = spokePoolClients[destinationChainId];
                    if (fillBlock.lt(destSpoke.deploymentBlock) || fillBlock.gt(destSpoke.latestBlockSearched)) {
                        deploymentBlock = destSpoke.deploymentBlock, latestBlockSearched = destSpoke.latestBlockSearched;
                        return [2, {
                                valid: false,
                                reason: "FillBlock (".concat(fillBlock, " out of SpokePool range [").concat(deploymentBlock, ", ").concat(latestBlockSearched, "]"),
                            }];
                    }
                    if (previousIdenticalRequests.gt(0)) {
                        return [2, { valid: false, reason: "Previous identical request exists" }];
                    }
                    fill = destSpoke.getFillsForRelayer(relayer).find(function (fill) {
                        return (fill.depositId === depositId
                            && fill.originChainId === originChainId
                            && fill.destinationChainId === destinationChainId
                            && fill.repaymentChainId === repaymentChainId
                            && fill.amount.eq(amount)
                            && fill.fillAmount.eq(amount)
                            && fill.realizedLpFeePct.eq(realizedLpFeePct)
                            && fill.blockNumber === fillBlock.toNumber());
                    });
                    if (!(0, TypeGuards_1.isDefined)(fill)) {
                        if (fillBlock.lt(destSpoke.eventSearchConfig.fromBlock)) {
                            throw new Error("Unimplemented: refund request fillBlock ".concat(fillBlock, " is older than spoke pool client from block, set a wider lookback"));
                        }
                        else {
                            return [2, { valid: false, reason: "Unable to find matching fill" }];
                        }
                    }
                    return [4, getMatchedDeposit(spokePoolClients, fill, ignoredDepositValidationParams)];
                case 1:
                    deposit = _a.sent();
                    if (!(0, TypeGuards_1.isDefined)(deposit)) {
                        return [2, { valid: false, reason: "Unable to find matching deposit" }];
                    }
                    try {
                        expectedRefundToken = hubPoolClient.getL2TokenForDeposit(deposit, repaymentChainId);
                        if (expectedRefundToken !== refundToken) {
                            return [2, { valid: false, reason: "Refund token does not map to expected refund token ".concat(refundToken) }];
                        }
                    }
                    catch (_b) {
                        return [2, { valid: false, reason: "Refund token unknown at HubPool block ".concat(deposit.quoteBlockNumber) }];
                    }
                    return [2, { valid: true, matchingFill: fill, matchingDeposit: deposit }];
            }
        });
    });
}
exports.refundRequestIsValid = refundRequestIsValid;
function getMatchingFlow(allValidatedFlows, targetFlow) {
    return allValidatedFlows === null || allValidatedFlows === void 0 ? void 0 : allValidatedFlows.find(function (_a) {
        var flow = _a.flow;
        return flow.depositId === targetFlow.depositId && flow.originChainId === targetFlow.originChainId;
    });
}
exports.getMatchingFlow = getMatchingFlow;
function getValidFillCandidates(chainId, spokePoolClients, filter, ignoredDepositValidationParams) {
    if (filter === void 0) { filter = {}; }
    if (ignoredDepositValidationParams === void 0) { ignoredDepositValidationParams = []; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var spokePoolClient, repaymentChainId, relayer, isSlowRelay, isCompleteFill, fromBlock, toBlock, fills;
        var _this = this;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    spokePoolClient = spokePoolClients[chainId];
                    (0, assert_1.default)((0, TypeGuards_1.isDefined)(spokePoolClient));
                    repaymentChainId = filter.repaymentChainId, relayer = filter.relayer, isSlowRelay = filter.isSlowRelay, isCompleteFill = filter.isCompleteFill, fromBlock = filter.fromBlock, toBlock = filter.toBlock;
                    return [4, (0, ArrayUtils_1.mapAsync)(spokePoolClient.getFills(), function (fill) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                            var deposit;
                            return tslib_1.__generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        if ((0, TypeGuards_1.isDefined)(fromBlock) && fromBlock > fill.blockNumber) {
                                            return [2, undefined];
                                        }
                                        if ((0, TypeGuards_1.isDefined)(toBlock) && toBlock < fill.blockNumber) {
                                            return [2, undefined];
                                        }
                                        if (((0, TypeGuards_1.isDefined)(repaymentChainId) && fill.repaymentChainId !== repaymentChainId) ||
                                            ((0, TypeGuards_1.isDefined)(relayer) && fill.relayer !== relayer) ||
                                            ((0, TypeGuards_1.isDefined)(isSlowRelay) && fill.updatableRelayData.isSlowRelay !== isSlowRelay)) {
                                            return [2, undefined];
                                        }
                                        if ((0, TypeGuards_1.isDefined)(isCompleteFill) && isCompleteFill !== fill.fillAmount.eq(fill.totalFilledAmount)) {
                                            return [2, undefined];
                                        }
                                        return [4, getMatchedDeposit(spokePoolClients, fill, ignoredDepositValidationParams)];
                                    case 1:
                                        deposit = _a.sent();
                                        if (deposit !== undefined) {
                                            return [2, tslib_1.__assign(tslib_1.__assign({}, fill), { matchedDeposit: deposit })];
                                        }
                                        else
                                            return [2, undefined];
                                        return [2];
                                }
                            });
                        }); })];
                case 1:
                    fills = (_a.sent()).filter(TypeGuards_1.isDefined);
                    return [2, fills];
            }
        });
    });
}
exports.getValidFillCandidates = getValidFillCandidates;
function getValidRefundCandidates(chainId, hubPoolClient, spokePoolClients, filter, ignoredDepositValidationParams) {
    if (filter === void 0) { filter = {}; }
    if (ignoredDepositValidationParams === void 0) { ignoredDepositValidationParams = []; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var spokePoolClient, fromBlock, toBlock;
        var _this = this;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    spokePoolClient = spokePoolClients[chainId];
                    (0, assert_1.default)((0, TypeGuards_1.isDefined)(spokePoolClient));
                    fromBlock = filter.fromBlock, toBlock = filter.toBlock;
                    return [4, (0, ArrayUtils_1.mapAsync)(spokePoolClient.getRefundRequests(), function (refundRequest) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                            var result, matchedDeposit;
                            return tslib_1.__generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        if ((0, TypeGuards_1.isDefined)(fromBlock) && fromBlock > refundRequest.blockNumber) {
                                            return [2, undefined];
                                        }
                                        if ((0, TypeGuards_1.isDefined)(toBlock) && toBlock < refundRequest.blockNumber) {
                                            return [2, undefined];
                                        }
                                        return [4, refundRequestIsValid(spokePoolClients, hubPoolClient, refundRequest, ignoredDepositValidationParams)];
                                    case 1:
                                        result = _a.sent();
                                        if (result.valid) {
                                            matchedDeposit = result.matchingDeposit;
                                            if (matchedDeposit === undefined) {
                                                throw new Error("refundRequestIsValid returned true but matchingDeposit is undefined");
                                            }
                                            return [2, tslib_1.__assign(tslib_1.__assign({}, refundRequest), { matchedDeposit: matchedDeposit })];
                                        }
                                        else {
                                            return [2, undefined];
                                        }
                                        return [2];
                                }
                            });
                        }); })];
                case 1: return [2, (_a.sent()).filter(function (refundRequest) { return refundRequest !== undefined; })];
            }
        });
    });
}
exports.getValidRefundCandidates = getValidRefundCandidates;
function serializeUBAClientState(ubaClientState) {
    return (0, JSONUtils_1.stringifyJSONWithNumericString)(ubaClientState);
}
exports.serializeUBAClientState = serializeUBAClientState;
function deserializeUBAClientState(serializedUBAClientState) {
    return Object.entries(serializedUBAClientState).reduce(function (acc, _a) {
        var _b;
        var chainId = _a[0], chainState = _a[1];
        if (typeof chainState !== "object") {
            throw new Error("Failed to parse chain state for chain ".concat(chainId));
        }
        return tslib_1.__assign(tslib_1.__assign({}, acc), (_b = {}, _b[chainId] = tslib_1.__assign(tslib_1.__assign({}, chainState), { bundles: Object.entries(chainState.bundles).reduce(function (acc, _a) {
                var _b;
                var tokenSymbol = _a[0], bundleStates = _a[1];
                if (!Array.isArray(bundleStates)) {
                    throw new Error("Failed to parse bundle states for token ".concat(tokenSymbol));
                }
                return tslib_1.__assign(tslib_1.__assign({}, acc), (_b = {}, _b[tokenSymbol] = bundleStates.map(function (bundleState) {
                    var deserializedUBAFeeConfig = deserializeUBAFeeConfig(bundleState.config);
                    return tslib_1.__assign(tslib_1.__assign({}, bundleState), { openingBalance: ethers_1.BigNumber.from(bundleState.openingBalance), openingIncentiveBalance: ethers_1.BigNumber.from(bundleState.openingIncentiveBalance), config: new UBAFeeConfig_1.default(deserializedUBAFeeConfig.baselineFee, deserializedUBAFeeConfig.balancingFee, deserializedUBAFeeConfig.balanceTriggerThreshold, deserializedUBAFeeConfig.lpGammaFunction, deserializedUBAFeeConfig.incentivePoolAdjustment, deserializedUBAFeeConfig.ubaRewardMultiplier), flows: bundleState.flows.map(deserializeModifiedUBAFlow) });
                }), _b));
            }, {}) }), _b));
    }, {});
}
exports.deserializeUBAClientState = deserializeUBAClientState;
function deserializeModifiedUBAFlow(serializedModifiedUBAFlow) {
    return {
        flow: deserializeBigNumberValues(serializedModifiedUBAFlow.flow),
        systemFee: deserializeBigNumberValues(serializedModifiedUBAFlow.systemFee),
        relayerFee: deserializeBigNumberValues(serializedModifiedUBAFlow.relayerFee),
        runningBalance: ethers_1.BigNumber.from(serializedModifiedUBAFlow.runningBalance),
        incentiveBalance: ethers_1.BigNumber.from(serializedModifiedUBAFlow.incentiveBalance),
        netRunningBalanceAdjustment: ethers_1.BigNumber.from(serializedModifiedUBAFlow.netRunningBalanceAdjustment),
    };
}
function deserializeUBAFeeConfig(serializedUBAFeeConfig) {
    var _a, _b, _c, _d, _e, _f;
    return {
        baselineFee: {
            default: ethers_1.BigNumber.from(serializedUBAFeeConfig.baselineFee.default),
            override: Object.entries((_a = serializedUBAFeeConfig.baselineFee.override) !== null && _a !== void 0 ? _a : {}).reduce(function (acc, _a) {
                var key = _a[0], value = _a[1];
                acc[key] = ethers_1.BigNumber.from(value);
                return acc;
            }, {}),
        },
        balancingFee: {
            default: serializedUBAFeeConfig.balancingFee.default.map(function (tuple) {
                return tuple.map(function (value) { return ethers_1.BigNumber.from(value); });
            }),
            override: Object.entries((_b = serializedUBAFeeConfig.balancingFee.override) !== null && _b !== void 0 ? _b : {}).reduce(function (acc, _a) {
                var key = _a[0], value = _a[1];
                acc[key] = value.map(function (tuple) { return tuple.map(function (value) { return ethers_1.BigNumber.from(value); }); });
                return acc;
            }, {}),
        },
        balanceTriggerThreshold: {
            default: {
                lowerBound: deserializeBigNumberValues(serializedUBAFeeConfig.balanceTriggerThreshold.default.lowerBound),
                upperBound: deserializeBigNumberValues(serializedUBAFeeConfig.balanceTriggerThreshold.default.upperBound),
            },
            override: Object.entries((_c = serializedUBAFeeConfig.balanceTriggerThreshold.override) !== null && _c !== void 0 ? _c : {}).reduce(function (acc, _a) {
                var key = _a[0], value = _a[1];
                acc[key] = {
                    lowerBound: deserializeBigNumberValues(value.lowerBound),
                    upperBound: deserializeBigNumberValues(value.upperBound),
                };
                return acc;
            }, {}),
        },
        lpGammaFunction: {
            default: serializedUBAFeeConfig.lpGammaFunction.default.map(function (tuple) {
                return tuple.map(function (value) { return ethers_1.BigNumber.from(value); });
            }),
            override: Object.entries((_d = serializedUBAFeeConfig.lpGammaFunction.override) !== null && _d !== void 0 ? _d : {}).reduce(function (acc, _a) {
                var key = _a[0], value = _a[1];
                acc[key] = value.map(function (tuple) { return tuple.map(function (value) { return ethers_1.BigNumber.from(value); }); });
                return acc;
            }, {}),
        },
        incentivePoolAdjustment: Object.entries((_e = serializedUBAFeeConfig.incentivePoolAdjustment) !== null && _e !== void 0 ? _e : {}).reduce(function (acc, _a) {
            var key = _a[0], value = _a[1];
            acc[key] = ethers_1.BigNumber.from(value);
            return acc;
        }, {}),
        ubaRewardMultiplier: Object.entries((_f = serializedUBAFeeConfig.ubaRewardMultiplier) !== null && _f !== void 0 ? _f : {}).reduce(function (acc, _a) {
            var key = _a[0], value = _a[1];
            acc[key] = ethers_1.BigNumber.from(value);
            return acc;
        }, {}),
    };
}
function deserializeBigNumberValues(obj) {
    if (obj === void 0) { obj = {}; }
    return Object.entries(obj).reduce(function (acc, _a) {
        var _b, _c;
        var key = _a[0], value = _a[1];
        try {
            var parsedBigNumber = ethers_1.BigNumber.from(value);
            return tslib_1.__assign(tslib_1.__assign({}, acc), (_b = {}, _b[key] = parsedBigNumber, _b));
        }
        catch (_d) {
            return tslib_1.__assign(tslib_1.__assign({}, acc), (_c = {}, _c[key] = value, _c));
        }
    }, {});
}
//# sourceMappingURL=UBAClientUtilities.js.map