"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SpokePoolEventsQueryService = void 0;
var tslib_1 = require("tslib");
var config_1 = require("../config");
var SpokePoolEventsQueryService = (function () {
    function SpokePoolEventsQueryService(chainId, provider, eventsQuerier, logger, depositorAddr) {
        this.chainId = chainId;
        this.provider = provider;
        this.eventsQuerier = eventsQuerier;
        this.logger = logger;
        this.depositorAddr = depositorAddr;
    }
    SpokePoolEventsQueryService.prototype.getEvents = function () {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var from, depositEvents, filledRelayEvents, speedUpDepositEvents, blockTimestampMap, to, start, _b, depositEventsSettled, filledRelayEventsSettled, speedUpDepositEventsSettled, end;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        depositEvents = [];
                        filledRelayEvents = [];
                        speedUpDepositEvents = [];
                        blockTimestampMap = {};
                        if (this.latestBlockSearched) {
                            from = this.latestBlockSearched + 1;
                        }
                        else {
                            from = (_a = config_1.clientConfig.spokePools[this.chainId].lowerBoundBlockNumber) !== null && _a !== void 0 ? _a : -1;
                        }
                        return [4, this.provider.getBlock("latest")];
                    case 1:
                        to = (_c.sent()).number;
                        if (from > to) {
                            this.logger.debug("[SpokePoolEventsQueryService]", "\uD83D\uDD34 chain ".concat(this.chainId, ": from ").concat(from, " > to ").concat(to));
                            return [2, {
                                    emittedFromChainId: this.chainId,
                                    depositEvents: [],
                                    filledRelayEvents: [],
                                    speedUpDepositEvents: [],
                                    blockTimestampMap: {},
                                }];
                        }
                        start = new Date();
                        return [4, Promise.allSettled([
                                this.eventsQuerier.getFundsDepositEvents(from, to, this.depositorAddr),
                                this.eventsQuerier.getFilledRelayEvents(from, to, this.depositorAddr),
                                this.eventsQuerier.getSpeedUpDepositEvents(from, to, this.depositorAddr),
                            ])];
                    case 2:
                        _b = _c.sent(), depositEventsSettled = _b[0], filledRelayEventsSettled = _b[1], speedUpDepositEventsSettled = _b[2];
                        depositEvents = depositEventsSettled.status === "fulfilled" ? depositEventsSettled.value : [];
                        filledRelayEvents = filledRelayEventsSettled.status === "fulfilled" ? filledRelayEventsSettled.value : [];
                        speedUpDepositEvents = speedUpDepositEventsSettled.status === "fulfilled" ? speedUpDepositEventsSettled.value : [];
                        end = new Date();
                        this.logger.debug("[SpokePoolEventsQueryService::getEvents]", "\uD83D\uDFE2 chain ".concat(this.chainId, ": ").concat(depositEvents.length, " FundsDeposited events, ").concat(filledRelayEvents.length, " FilledRelayEvents, ").concat(speedUpDepositEvents.length, " RequestedSpeedUpDeposit events, blocks: ").concat(from, " -> ").concat(to, ", ").concat((end.valueOf() - start.valueOf()) / 1000, " seconds"));
                        start = new Date();
                        return [4, this.getBlocksTimestamp(tslib_1.__spreadArray(tslib_1.__spreadArray([], depositEvents, true), speedUpDepositEvents, true))];
                    case 3:
                        blockTimestampMap = _c.sent();
                        end = new Date();
                        this.logger.debug("[SpokePoolEventsQueryService::getEvents]", "\uD83D\uDFE2 chain ".concat(this.chainId, ": fetched block numbers in ").concat((end.valueOf() - start.valueOf()) / 1000, " seconds"));
                        this.latestBlockSearched = to;
                        return [2, {
                                emittedFromChainId: this.chainId,
                                depositEvents: depositEvents,
                                filledRelayEvents: filledRelayEvents,
                                speedUpDepositEvents: speedUpDepositEvents,
                                blockTimestampMap: blockTimestampMap,
                            }];
                }
            });
        });
    };
    SpokePoolEventsQueryService.prototype.getBlocksTimestamp = function (events) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var uniqueBlockNumbers, uniqueBlockNumbersList, blocksChunks, blocks, _i, blocksChunks_1, blocksChunk, newBlocks, timestamps, blockTimestampMap;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        uniqueBlockNumbers = events.reduce(function (acc, event) {
                            var _a;
                            return tslib_1.__assign(tslib_1.__assign({}, acc), (_a = {}, _a[event.blockNumber] = true, _a));
                        }, {});
                        uniqueBlockNumbersList = Object.keys(uniqueBlockNumbers).map(function (blockNumber) { return parseInt(blockNumber); });
                        this.logger.debug("[getBlocksTimestamp]", "\uD83D\uDFE2 chain ".concat(this.chainId, ": fetching ").concat(uniqueBlockNumbersList.length, " blocks"));
                        blocksChunks = this.getArrayChunks(uniqueBlockNumbersList);
                        blocks = [];
                        _i = 0, blocksChunks_1 = blocksChunks;
                        _a.label = 1;
                    case 1:
                        if (!(_i < blocksChunks_1.length)) return [3, 4];
                        blocksChunk = blocksChunks_1[_i];
                        return [4, Promise.all(blocksChunk.map(function (blockNumber) { return _this.provider.getBlock(blockNumber); }))];
                    case 2:
                        newBlocks = _a.sent();
                        blocks.push.apply(blocks, newBlocks);
                        _a.label = 3;
                    case 3:
                        _i++;
                        return [3, 1];
                    case 4: return [4, Promise.all(blocks.map(function (block) { return block.timestamp; }))];
                    case 5:
                        timestamps = _a.sent();
                        blockTimestampMap = uniqueBlockNumbersList.reduce(function (acc, blockNumber, idx) {
                            var _a;
                            return (tslib_1.__assign(tslib_1.__assign({}, acc), (_a = {}, _a[blockNumber] = timestamps[idx], _a)));
                        }, {});
                        return [2, blockTimestampMap];
                }
            });
        });
    };
    SpokePoolEventsQueryService.prototype.getArrayChunks = function (array, chunkSize) {
        if (chunkSize === void 0) { chunkSize = 50; }
        return Array(Math.ceil(array.length / chunkSize))
            .fill([])
            .map(function (_, index) { return index * chunkSize; })
            .map(function (begin) { return array.slice(begin, begin + chunkSize); });
    };
    return SpokePoolEventsQueryService;
}());
exports.SpokePoolEventsQueryService = SpokePoolEventsQueryService;
//# sourceMappingURL=SpokePoolEventsQueryService.js.map