import { BigNumber } from "ethers";
import { UBA_CONFIG_ONCHAIN_SCHEMA } from "./superstructs";
/**
 * Type guard for UBAOnChainConfigType
 * @param input An unknown value that may or may not be a UBAOnChainConfigType
 * @returns True if the input is a UBAOnChainConfigType, false otherwise
 */
export function isUBAOnChainConfig(input) {
    return UBA_CONFIG_ONCHAIN_SCHEMA.is(input);
}
/**
 * Parses a UBAOnChainConfigType into a UBAParsedConfigType
 * @param config The UBAOnChainConfigType to parse
 * @returns The parsed UBAParsedConfigType
 * @throws If the config is undefined
 * @throws If the config is not a UBAOnChainConfigType
 * @throws If the config is not valid
 */
export function parseUBAConfigFromOnChain(config) {
    var _a, _b;
    // Check that the config is defined
    if (!isUBAOnChainConfig(config)) {
        // If it's not, throw an error
        throw new Error("No config provided");
    }
    return {
        incentivePoolAdjustment: Object.fromEntries(Object.entries((_a = config.incentivePoolAdjustment) !== null && _a !== void 0 ? _a : {}).map(function (_a) {
            var key = _a[0], value = _a[1];
            return [key, BigNumber.from(value)];
        })),
        ubaRewardMultiplier: Object.fromEntries(Object.entries((_b = config.ubaRewardMultiplier) !== null && _b !== void 0 ? _b : {}).map(function (_a) {
            var key = _a[0], value = _a[1];
            return [key, BigNumber.from(value)];
        })),
        alpha: Object.fromEntries(Object.entries(config.alpha).map(function (_a) {
            var key = _a[0], value = _a[1];
            return [key, BigNumber.from(value)];
        })),
        gamma: Object.fromEntries(Object.entries(config.gamma).map(function (_a) {
            var key = _a[0], value = _a[1];
            return [
                key,
                value.map(function (tuple) { return tuple.map(function (value) { return BigNumber.from(value); }); }),
            ];
        })),
        omega: Object.fromEntries(Object.entries(config.omega).map(function (_a) {
            var key = _a[0], value = _a[1];
            return [
                key,
                value.map(function (tuple) { return tuple.map(function (value) { return BigNumber.from(value); }); }),
            ];
        })),
        rebalance: Object.fromEntries(Object.entries(config.rebalance).map(function (_a) {
            var key = _a[0], _b = _a[1], threshold_lower = _b.threshold_lower, threshold_upper = _b.threshold_upper, target_lower = _b.target_lower, target_upper = _b.target_upper;
            var upperExists = threshold_upper !== undefined && target_upper !== undefined;
            var lowerExists = threshold_lower !== undefined && target_lower !== undefined;
            return [
                key,
                {
                    threshold_lower: lowerExists ? BigNumber.from(threshold_lower) : undefined,
                    target_lower: lowerExists ? BigNumber.from(target_lower) : undefined,
                    threshold_upper: upperExists ? BigNumber.from(threshold_upper) : undefined,
                    target_upper: upperExists ? BigNumber.from(target_upper) : undefined,
                },
            ];
        })),
    };
}
//# sourceMappingURL=ConfigStoreParsingUtilities.js.map