import { isUbaInflow, outflowIsFill, } from "../interfaces";
export var FILL_DEPOSIT_COMPARISON_KEYS = [
    "amount",
    "originChainId",
    "relayerFeePct",
    "realizedLpFeePct",
    "depositId",
    "depositor",
    "recipient",
    "destinationChainId",
    "destinationToken",
    "message",
];
export function getTokenSymbolForFlow(flow, chainId, hubPoolClient) {
    var _a, _b, _c;
    var tokenSymbol;
    if (isUbaInflow(flow)) {
        if (chainId !== flow.originChainId) {
            throw new Error("ChainId mismatch on chain ".concat(flow.originChainId, " deposit ").concat(flow.depositId, " (").concat(chainId, " != ").concat(flow.originChainId, ")"));
        }
        tokenSymbol = (_a = hubPoolClient.getTokenInfo(flow.originChainId, flow.originToken)) === null || _a === void 0 ? void 0 : _a.symbol;
    }
    else if (outflowIsFill(flow)) {
        if (chainId !== flow.destinationChainId) {
            throw new Error("ChainId mismatch on chain ".concat(flow.destinationChainId, " fill for chain ").concat(flow.originChainId, " deposit ").concat(flow.depositId, " (").concat(chainId, " != ").concat(flow.destinationChainId, ")"));
        }
        tokenSymbol = (_b = hubPoolClient.getTokenInfo(flow.destinationChainId, flow.destinationToken)) === null || _b === void 0 ? void 0 : _b.symbol;
    }
    else if (chainId !== flow.repaymentChainId) {
        if (chainId !== flow.repaymentChainId) {
            throw new Error("ChainId mismatch on chain ".concat(flow.repaymentChainId, " for chain ").concat(flow.originChainId, " deposit ").concat(flow.depositId, " (").concat(chainId, " != ").concat(flow.repaymentChainId, ")"));
        }
        tokenSymbol = (_c = hubPoolClient.getTokenInfo(flow.repaymentChainId, flow.refundToken)) === null || _c === void 0 ? void 0 : _c.symbol;
    }
    return tokenSymbol;
}
export function filledSameDeposit(fillA, fillB) {
    return (fillA.depositId === fillB.depositId &&
        fillA.originChainId === fillB.originChainId &&
        fillA.amount.eq(fillB.amount) &&
        fillA.destinationChainId === fillB.destinationChainId &&
        fillA.relayerFeePct.eq(fillB.relayerFeePct) &&
        fillA.recipient === fillB.recipient &&
        fillA.depositor === fillB.depositor);
}
// Ensure that each deposit element is included with the same value in the fill. This includes all elements defined
// by the depositor as well as the realizedLpFeePct and the destinationToken, which are pulled from other clients.
export function validateFillForDeposit(fill, deposit, fillFieldsToIgnore) {
    if (fillFieldsToIgnore === void 0) { fillFieldsToIgnore = []; }
    if (deposit === undefined) {
        return false;
    }
    // Note: this short circuits when a key is found where the comparison doesn't match.
    // TODO: if we turn on "strict" in the tsconfig, the elements of FILL_DEPOSIT_COMPARISON_KEYS will be automatically
    // validated against the fields in Fill and Deposit, generating an error if there is a discrepency.
    return FILL_DEPOSIT_COMPARISON_KEYS.every(function (key) {
        var _a;
        if (fillFieldsToIgnore.includes(key)) {
            return true;
        }
        return fill[key] !== undefined && fill[key].toString() === ((_a = deposit[key]) === null || _a === void 0 ? void 0 : _a.toString());
    });
}
//# sourceMappingURL=FlowUtils.js.map